// Copyright (C) 2000 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "bayonnedb.h"
#include <getopt.h>

Database *database = NULL;
Driver *Driver::first = NULL;
Server *Server::first = NULL;

Driver::Driver()
{
	next = first;
	first = this;
}

void initialize(void)
{
	Driver *driver = Driver::first;

	while(driver)
	{
		driver->Initial();
		driver = driver->next;
	}
}

void preview(unsigned char *msg, unsigned len)
{
	Driver *driver = Driver::first;

	while(driver)
	{
		driver->Preview(msg, len);
		driver = driver->next;
	}
}

int dispatch(unsigned char *msg, unsigned len)
{
	Driver *driver = Driver::first;
	int rlen;

	while(driver)
	{
		if(0 != (rlen = driver->Accept(msg, len)))
			return rlen;
		driver = driver->next;
	}
	return -1;
}

void finalize(int excode)
{
	Driver *driver = Driver::first;

	while(driver)
	{
		driver->Final(excode);
		driver = driver->next;
	}
}

void scheduler(void)
{
	Driver *driver;
	time_t now;
	time_t last;
	unsigned interval = keyserver.getInterval();

	time(&last);
	for(;;)
	{
		time(&now);
		if(now - last < interval)
			ccxx_sleep((interval - (now - last)) * 1000);
		else
			ccxx_sleep(interval * 1000);
		time(&last);
		driver = Driver::first;
		while(driver)
		{
			driver->Schedule();
			driver = driver->next;
		}
	}
}

void startServers(void)
{
	Server *server = Server::first;
	char *name;

	while(server)
	{
		server->Start();
		name = server->getName();
		if(name)
			slog(SLOG_INFO) << name << ": starting..." << endl;
		server = server->next;
	}
}

void stopServers(void)
{
	Server *server = Server::first;
	char *name;

	while(server)
	{
		name = server->getName();
		if(name)
			slog(SLOG_INFO) << name << ": stoping..." << endl;

		server->Stop();
		server = server->next;
	}
}

